"""The public API for the preferences."""

# The global PreferenceManager instance
from .preference_manager import preference_manager
from .bindings import set_scene_preferences, get_scene_preferences
